/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: vc_commands.c,v 1.4 2002/06/23 23:27:52 mickey Exp $ */

#include <vchat.h>
#include <proto_vchat.h>

#include <netdb.h>
#include <arpa/inet.h>

/*** Externals ***/

extern VCONN sconn;
extern VP vp;
extern ED *ed;
extern ULIST_ITEM *ulist_base;
extern int ulist_cnt;

/*** Code ***/

void handle_cmd(char *cmd)
{
	char *arg = &(cmd[2]);

	if(cmd[2] == ' ')
		arg = &cmd[3];

	switch(cmd[1])
	{
		case 'a':	action(arg);
				break;
		case 'A':	aclcmd(arg);
				break;
		case 'B':	broadcast(arg);
				break;
		case 'c':	chchan(arg);
				BZERO(&ed->buffer, EDBUFSIZE+1);
				ed->endpos = ed->curpos = ed->margin = (char *) &ed->buffer;
				ed->prompt = NULL;
				ed->offset = 0;
				ed->maxlen = EDBUFSIZE;
				vsleep(0, 650000);
				break;
		case 'C':	addnick(arg);
				break;
		case 'D':	delnick(arg);
				break;
		case 'F':	fixchan(arg);
				break;
		case 'G':	chserver(arg);
				break;
		case 'h':	help();
				break;
		case 'H':	ext_help();
				break;
		case 'i':	invite(arg);
				break;
		case 'I':	clientinfo(arg);
				break;
		case 'K':	killclient(arg);
				break;
		case 'l':	lastlog(arg);
				break;
		case 'L':	logcmd(arg);
				break;
		case 'm':	prv_msg(arg);
				break;
		case 'M':	modify_permissions(arg);
				break;
		case 'n':	chnick(arg);
				break;
		case 'N':	svdnsopt(arg);
				break;
		case 'o':	clientopts(arg);
				break;
		case 'O':	serveropts(arg);
				break;
		case 'p':	snd_serv(CMD_PAGE, arg, strlen(arg));
				break;
		case 'P':	chpasswd(arg);
				break;
		case 'Q':	querynick(arg);
				break;
		case 'r':	screen_init();
				break;
		case 'R':	rtcmp(arg);
				break;
		case 's':	ulist(arg);
				break;
		case 'S':	serverstatus(arg);
				break;
		case 't':	snd_serv(CMD_CHANLIST, 0, 0);
				break;
		case 'T':	chtopic(arg);
				break;
		case 'v':	version_info();
				break;
		case 'W':	wakeup(arg);
				break;
		case 'X':	exclude(arg);
				break;
		default:	cprintf(chat, "* No such command - type \"%s.h%s\" to get help.",
					S_FG_GRN, S_OFF);
				break;
	}
}

/*******************
 * Logfile command *
 *******************/

void logcmd(char *dat)
{
	char	*fl	= NULL;
	char	*op	= "open";
	char	*cl	= "close";

	if(dat == NULL || strlen(dat) == 0)
		goto status;

	if((fl = strchr(dat, ' ')) != NULL)
		*fl++ = '\0';
	else
		fl = "";

	str_lower(dat);

	if(strlen(dat) == 0)
		goto status;

	if(strstr(op, dat) == op)
	{
		if(strlen(fl) == 0)
		{
			cprintf(chat, "* No log filename specified - using default name 'vchat.log'");
			fl = "vchat.log";
		}

		if((vp.logfl = fopen(fl, "w")) == NULL)
		{
			cprintf(chat, "* Failed to open logfile '%s' (%s)",
				fl, strerror(errno));
		}
		else
		{
			sprintf(vp.logfile, "%s", fl);
			vp.logswitch = 0x1;
			vp.logcnt = 0;
			update_flags();
			cprintf(chat, "* Logfile '%s' opened", vp.logfile);
		}

		return;
	}
	else if(strstr(cl, dat) == cl)
	{
		if(vp.logswitch != 1)
		{
			cprintf(chat, "* Logfile has not been opened");
		}
		else
		{
			fclose(vp.logfl);

			cprintf(chat, "* Logfile '%s' closed [%d bytes]",
				vp.logfile, vp.logcnt);

			vp.logfl = NULL;
			sprintf(vp.logfile, "%c", '\0');
			vp.logswitch = vp.logcnt = 0;
			update_flags();
		}

		return;
	}

	/*** Display logfile status ***/

status:

	if(vp.logswitch == 1)
	{
		cprintf(chat, "* Logfile '%s' open [%d bytes]",
			vp.logfile, vp.logcnt);
	}
	else
	{
		cprintf(chat, "* Logfile has not been opened");
	}
}

/**********************************
 * Change to another VChat server *
 **********************************/

void chserver(char *dat)
{
	u_char		nicksav[VPNICKSIZE+1];
	u_char		nickbuf[VPNICKSIZE+1];
	u_char		prompt[64];
	struct in_addr	addr;
	struct hostent	*ht;
	VCONN		remconn;
	int		ret;
	char		*sp;
	u_char		*nickptr;
	u_short		vs_port;
	u_char		nickchg	= 0;

	/*
	 * See if the nickname was given on the commandline
	 */

	BZERO(nickbuf, VPNICKSIZE+1);

	if((nickptr = (u_char *)strchr(dat, ' ')) != NULL)
	{
		*nickptr++ = '\0';
	}

	/*
	 * See if a server port number was specified or take default port
	 */

	if((sp = (char *)strchr(dat, ':')) != NULL)
	{
		*sp++ = '\0';

		vs_port = atoi(sp);
	}	
	else
	{
		vs_port = SERVER_PORT;
	}

	/*
	 * Lookup hostname
	 */

	if((ht = gethostbyname(dat)) == NULL)
	{
#if HAVE_HSTRERROR
		cprintf(chat, "* No address associated with name '%s' (%s%s%s)",
			dat, S_FG_RED, hstrerror(h_errno), S_OFF);
#else
		cprintf(chat, "* No address associated with name '%s' (%sUnknown host%s)",
			dat, S_FG_RED, S_OFF);
#endif
		return;
	}

	BCOPY(ht->h_addr_list[0], &addr, sizeof(struct in_addr));

	if(!memcmp(&vp.sv_ip, &addr, sizeof(struct in_addr)))
	{
		cprintf(chat, "* You are already connected to %s%s%s",
			S_FG_GRN, sconn.host, S_OFF);
		return;
	}

	/*
	 * If no nickname was specified on the commandline, request it now
	 */

	if(!nickptr)
	{
		/*
		 * Request nickname from user defaulting to current nickname
		 */

		sprintf(prompt, "%sNickname%s%s:%s ",
			S_BOLD, S_OFF, S_BLINK, S_OFF);

		clear_ed(ed);

		sprintf(ed->buffer, "%s", vp.nick);
		ed->endpos = ed->curpos = ed->buffer + strlen(vp.nick);

		if(input_line(prompt, VPNICKSIZE, 0) == 0)
		{
			clear_input();
			cprintf(chat, "* Aborted");
			return;
		}

		clear_input();
		nickptr = ed->buffer;
	}

	strncpy(nickbuf, nickptr, VPNICKSIZE);

	/*
	 * Request password
	 */

	sprintf(prompt, "%sPassword%s%s:%s ",
		S_BOLD, S_OFF, S_BLINK, S_OFF);

	input_line(prompt, VPPASSWDSIZE, (ED_RST_CTX | ED_NO_ECHO));

	clear_input();

	/*
	 * Save the current nickname and change to the specified nickname.
	 * If the connection to the new server fails, reset the current
	 * nickname to the saved nickname.
	 */

	nickchg = (u_char)strncmp(vp.nick, nickbuf, VPNICKSIZE);

	if(nickchg)
	{
		BZERO(nicksav, VPNICKSIZE+1);
		strncpy(nicksav, vp.nick, VPNICKSIZE);

		strncpy(vp.nick, nickbuf, VPNICKSIZE);
	}

	/*
	 * Try to establish server connection
	 */

	cprintf(chat, "\n* Trying %s:%d...", inet_ntoa(addr), vs_port);

	vlock(LOCK, (V_TIMER | V_WINCH));
	async_io(0);

	ret = connect_server(&remconn, &addr, vs_port, VSERVER_MAGIC, ed->buffer);

	clear_ed(ed);

	switch(ret)
	{
		case -1:	if(nickchg)
					strncpy(vp.nick, nicksav, VPNICKSIZE);

				cprintf(chat, "* Connect failed (%s%s%s)",
					S_FG_RED, strerror(errno), S_OFF);
				async_io(1);
				vlock(UNLOCK, (V_TIMER | V_WINCH));
				return;

		case CONN_OK:	break;

		default:	if(nickchg)
					strncpy(vp.nick, nicksav, VPNICKSIZE);

				cprintf(chat, "* %s", v_error(ret));
				async_io(1);
				vlock(UNLOCK, (V_TIMER | V_WINCH));
				return;
	}

	/*
	 * Send the signoff command to the server
	 * and close the old server reference
	 */

	cprintf(chat, "* Closing connection to server %s%s%s",
		S_FG_GRN, sconn.host, S_OFF);

	snd_serv(CMD_SIGNOFF, NULL, 0);
	close(sconn.fd);

	/*
	 * Copy the new server reference
	 */

	BCOPY(&remconn, &sconn, sizeof(VCONN));
	vp.sv_ip = addr;
	vp.sv_port = vs_port;

	/*
	 * Update server status info
	 */

	if(nickchg)
		alter_status(STAT_NICK, "%s ", vp.nick);

	alter_status(STAT_SERVER, "%s ", sconn.host);
	update_status(0);

	/*
	 * Re-enable the reception of server messages
	 */

	rcv_sv_msg();

	async_io(1);		
	vlock(UNLOCK, (V_TIMER | V_WINCH));

	vsleep(1,0);
}
