/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: vc_subr.c,v 1.2 2001/03/19 14:54:04 mickey Exp $ */

#include <vchat.h>
#include <proto_vchat.h>

#include <ctype.h>
#include <netdb.h>
#include <pwd.h>
#include <signal.h>
#include <arpa/inet.h>

#if HAVE_TERMIOS_H
#  include <termios.h>
#endif

#include <sys/ioctl.h>

#ifdef HAVE_RPLAY
#  include <rplay.h>
#endif

/******* Globals ********/

signed long inichan = 0;

extern	char *prog_name;
extern	char *optarg;

extern	int optind;
extern	int opterr;
extern	int optopt;
extern	int errno;

extern	VCONN	sconn;
extern	VTCAP	vtcap;
extern	VP	vp;
extern	ED	main_ed;

#if SHELLMODE
extern	ED	cmd_ed;
#endif

/*
 * int init(void);
 *
 * Check certain operational parameters like if we are running on
 * a real tty etc, Setup operating environment, Lookout for VSERVER
 * and NICK environment settings, Get user entry from passwd database.
 * Returns 0 if successfull, -1 in case of failure.
 */

int init(void)
{
	struct passwd *pwf;
	struct hostent *ht;
	char *nk;

	/*
	 * Set unbuffered output,
	 * so we wont have to mess
	 * up with lots of fflush'es.
	 */

	setvbuf(stdin, NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);

	/*
	 * Check, if we are really
	 * on a terminal line.
	 */

	if(!isatty(0) || !isatty(1))
	{
		fprintf(stderr, "%s: enter vchat interactive only.\n", prog_name);
		return(-1);
	}

	/*
	 * Set the default port
	 * to connect vserver at.
	 */

	vp.sv_port = SERVER_PORT;

	/*
	 * Set some nice deafults
	 */

	vp.bs_mode		= 1;	/* Blockscroll on backspace	*/
	vp.sc_mode		= 0;	/* Softscroll on cursor keys	*/
	vp.word_wrap		= 1;	/* Word-wrap on			*/
	vp.indent		= 1;	/* Indentation on		*/
	vp.hicolor		= 1;	/* Hicolor on			*/
	vp.cursordelay		= 3;	/* Cursordelay 3 secs		*/
	vp.videosnake		= 2;	/* Videosnake after 2 mins	*/
	vp.cmdcontext_invalid	= 1;	/* Command context yet invalid	*/
	vp.cmdmode		= 0;	/* Command mode not yet active	*/

	vp.history_lines = HISTORY_LINES; /* Command history lines	*/

	sprintf(vp.logfile, "%c", '\0');
	vp.logfl	= NULL;
	vp.logswitch	= 0;

	/*
	 * Initialize any input contexts needed
	 */

	clear_ed(&main_ed);
#if SHELLMODE
	clear_ed(&cmd_ed);
#endif
	/*
	 * Try to determine the terminal
	 * emulation currently in use
	 */

	if((vp.emulation = (char *) getenv("TERM")) == NULL)
		vp.emulation = "unknown";

	/*
	 * get user entry from
	 * passwd file
	 */

	if((pwf = getpwuid(getuid())) == NULL)
	{
		perror(prog_name);
		return(-1);
	}	
	else
	{
		strcpy(vp.user, pwf->pw_name);
		strcpy(vp.home, pwf->pw_dir);
		endpwent();
	}

	/*
	 * Look out for nickname
	 * environment settings
	 */

	if(!(nk = (char *)getenv("NICK")))
		strcpy(vp.nick, vp.user);
	else
		strcpy(vp.nick, nk);

	/*
	 * Ok, let's see if we got a
	 * server environment entry.
	 */

	if(!(nk = (char *)getenv("VSERVER")))
	{
		vp.sv_ip.s_addr = inet_addr("127.0.0.1");
	}
	else
	{
		if((ht = gethostbyname(nk)) == NULL)
		{
#if HAVE_HERROR
			herror(nk);
#else
			fprintf(stderr, "No address associated with name '%s'\n", nk);
#endif
			return(-1);
		}
		else
		{
			BCOPY(ht->h_addr, &vp.sv_ip, ht->h_length);
		}
	}

	return(0);
}

/*
 * int get_param(int argc, char *argv[]);
 *
 * Process commandline arguments given to vchat.
 * Returns 0 if successfull, -1 in case of failure.
 */

int get_param(int argc, char *argv[])
{
	int opt;
	struct hostent *ht;

	char optstr[] = ":c:n:p:s:l";

	while((opt = getopt(argc, argv, optstr)) != -1)
	{
		switch(opt)
		{
			case 'c':	inichan = strtol(optarg, NULL, 0);
					break;
			case 'n':	strcpy(vp.nick, optarg);
					break;
			case 'p':	vp.sv_port = atoi(optarg);
					break;
			case 's':	if((ht = gethostbyname(optarg)) == NULL)
					{
#if HAVE_HERROR
						herror(optarg);
#else
						fprintf(stderr, "%s: hostname lookup failure\n", optarg);
#endif
						return(-1);
					}
					else
					{
						BCOPY(ht->h_addr, &vp.sv_ip, ht->h_length);
					}
					break;
			case ':':	usage("Option -%c requires an argument.", optopt);
					return(-1);
			case '?':	usage("Unrecognized option: -%c", optopt);
					return(-1);
		}
	}

	return(0);
}

/*
 * void vquit(char *fmt, ...);
 *
 * Exit from the program with the specified message.
 */

void vquit(char *fmt, ...)
{
	va_list ap;

#if HAVE_SIGACTION
	struct sigaction sa;
#else
	struct sigvec sv;
#endif
	va_start(ap, fmt);

	/*
	 * Close client logfile (if any)
	 */

	if(vp.logswitch)
	{
		fclose(vp.logfl);
		vp.logswitch = 0;
	}

	/*
	 * Shutdown signal handlers
	 */

	v_io_end();
	v_timer_end();
	v_winch_end();

	/*
	 * Restore SIGINT and SIGTSTP to their defaults
	 */

#if HAVE_SIGACTION
	sa.sa_handler = SIG_DFL;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;

	sigaction(SIGINT, &sa, NULL);
	sigaction(SIGTSTP, &sa, NULL);
#else
	sv.sv_handler = SIG_DFL;
	sigemptyset(&sv.sv_mask);
	sv.sv_flags = 0;

	sigvec(SIGINT, &sv, NULL);
	sigvec(SIGTSTP, &sv, NULL);
#endif

	/*
	 * Free buffers used by the command history
	 */

	history_exit();

	/*
	 * Sleep a while...
	 */

	vsleep(0, 250000);

	/*
	 * Close connection to remote VChat server
	 */

	close(sconn.fd);

	/*
	 * Reset TTY key mode
	 */

	keymode(0);

	/*
	 * Output terminal reset sequence (if any)
	 */

	if(vtcap.term_reset)
		tputs(vtcap.term_reset, 1, (void *)outc);

	printf("%c[0;0r", 27);

	/*
	 * Move to the bottom of the screen and output
	 * program termination string.
	 */

 	mv(1, vtcap.rows);

	tty_mode(0);

	vprintf(fmt, ap);
	printf("\r\n\r\n");

	va_end(ap);

	/*
	 * Exit the program with zero return status
	 */

	exit(0);
}

/*
 * SIZE *getsize(void);
 *
 * Determines the current terminal size in characters.
 */

SIZE *getsize(void)
{
	static SIZE sz;

#ifdef TIOCGWINSZ
	struct winsize tsiz;

	ioctl(0, TIOCGWINSZ, &tsiz);

	sz.x = (tsiz.ws_col > 0) ? tsiz.ws_col : 0;
	sz.y = (tsiz.ws_row > 0) ? tsiz.ws_row : 0;
#else
  #ifdef TIOCGSIZE
	struct ttysize tsiz;

	ioctl(0, TIOCGSIZE, &tsiz);

	sz.x = (tsiz.ts_cols  > 0) ? tsiz.ts_cols  : 0;
	sz.y = (tsiz.ts_lines > 0) ? tsiz.ts_lines : 0;
  #else
	#error YOU BLEW IT! Cannot determine how to get terminal window size
  #endif
#endif

	return(&sz);
}

/*      
 * void ring_my_bell(char *dat, size_t size);
 * 
 * Outputs the name of the user specified by the parameter <dat> and <size>
 * followed by the word "DING!" in the users conference window and rings
 * the terminal bell. This function is called from rcv_sv_msg() whenever
 * a MSG_WAKEUP message is received from the remote conference server.
 */

void ring_my_bell(char *dat, size_t size)
{
	u_char name[VPNICKSIZE+1];

	BZERO(name, VPNICKSIZE+1);
	BCOPY(dat, name, (size > VPNICKSIZE) ? VPNICKSIZE : size);

#ifdef HAVE_RPLAY

	RPLAY *rpcmd;
	int rpl;

	if((rpl = rplay_open("deadline.snafu.de")) == -1)
	{
		cprintf(chat, "* rplay_open failed\n");
	}
	else
	{
		if((rpcmd = rplay_create(RPLAY_PLAY)) == NULL)
		{
			cprintf(chat, "* rplay_create failed\n");
		}
		else
		{
			rplay_set(rpcmd, RPLAY_INSERT, 0,
				RPLAY_SOUND, "ding",
				RPLAY_VOLUME, 200, NULL);

			rplay(rpl, rpcmd);

			rplay_destroy(rpcmd);
		}

		rplay_close(rpl);
	}
#else
	putchar(BEL);
#endif

	cprintf(chat, "(%s%s%s) %s%sDING!%s", S_FG_CYN, name, S_OFF,
		S_BOLD, S_BLINK, S_OFF);
}

/*
 * int read_string(u_char *buf, size_t len, char doecho);
 *
 * Read a string from the users terminal into the buffer
 * specified by <buf> with at most <len> bytes. If <doecho>
 * is not 0 then output will be echoed to the users terminal,
 * otherwise output will be suppressed. The return value is
 * the number of bytes placed into the buffer <buf>.
 */

int read_string(u_char *buf, size_t len, char doecho)
{
	char	c;
	u_char	cnt	= 0;

	for(;;)
	{
		/*
		 * Read a character from the users terminal
		 */

		c = getchar();

		/*
		 * Process character
		 */

		if(c == EOF || c == EOT)
		{
			BZERO(buf, len);
			return(0);
		}
		if(c == '\r' || c == '\n')
		{
			break;
		}
		else if(iscntrl((int)c))
		{
			switch(c)
			{
				case DEL:
				case BS:	if(cnt)
						{
							*(--buf) = '\0';
							--cnt;

							if(doecho)
								printf("%c%c%c", BS, ' ', BS);
						}
						break;

				case NAK:	while(cnt)
						{
							*(--buf) = '\0';
							--cnt;

							if(doecho)
								printf("%c%c%c", BS, ' ', BS);
						}
						break;
			}
		}
		else if(cnt < len)
		{
			*buf++ = c;
			++cnt;

			if(doecho)
				putchar(c);
		}
	}

	*buf = '\0';

	return(cnt);
}
