/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: vc_tty.c,v 1.2 2001/03/19 14:54:04 mickey Exp $ */

#include <vchat.h>
#include <proto_vchat.h>

#if HAVE_TERMIOS_H
  #include <termios.h>
#else
  #include <sys/ioctl.h>
#endif

/*** Code ***/

void tty_mode(int mode)
{
#if HAVE_TERMIOS_H
	static int saved = 0;
	static struct termios ttysave;
	struct termios ttyparam;

	switch(mode)
	{
		case 1:		tcgetattr(0, &ttysave);
				saved = 0xff;
				tcgetattr(0, &ttyparam);
				ttyparam.c_lflag &= ~(ECHO | ICANON);
				ttyparam.c_iflag &= ~ISTRIP;
  #if defined(XTABS)
				ttyparam.c_oflag &= ~XTABS;
  #elif defined(OXTABS)
				ttyparam.c_oflag &= ~OXTABS;
  #endif
				ttyparam.c_cc[VMIN] = 1;
				ttyparam.c_cc[VTIME] = 0;
				tcsetattr(0, TCSAFLUSH, &ttyparam);
				break;
		default:	if(saved)
					tcsetattr(0, TCSANOW, &ttysave);
				break;
	}
#else  /*** SGTTY ***/

	struct sgttyb term;
	static struct sgttyb termsav;
	static int saved = 0;

	switch(mode)
	{
		case 1:		ioctl(0, TIOCGETP, &termsav);
				saved = 0xff;
				ioctl(0, TIOCGETP, &term);

				term.sg_flags |= (CBREAK | PASS8OUT | PASS8);
				term.sg_flags &= ~(ECHO | XTABS);

				ioctl(0, TIOCSETP, &term);

				break;
		default:	if(saved)
					ioctl(0, TIOCSETP, &termsav);
				break;
	}
#endif
}
