/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: vc_vchatrc.c,v 1.2 2001/03/19 14:54:04 mickey Exp $ */

#include <vchat.h>
#include <proto_vchat.h>

#include <netdb.h>
#include <sys/file.h>
#include <arpa/inet.h>

/**** Globals ****/

int fx;

extern VP vp;

void vchatrc_load(void)
{
	FILE *vf;
	char *test;
	char fname[256];
	char tmp[256];

	sprintf(fname, "%s/.vchatrc", vp.home);

	if(access(fname, R_OK))
	{
		/*
		 * No .vchatrc available or
		 * .vchatrc not readable.
		 */
		
		return;
	}

	if((vf = fopen(fname, "r")) == NULL)
	{
		printf("\nFailed to open %s - ignored.\n", fname);
		return;
	}

	printf("\nReading %s ", fname);

	fx = 0xff;

	while(fgets(tmp, 255, vf) != NULL)
	{
		if((test = (char *)strrchr(tmp, '\n')) != NULL)
			*test = '\0';

		if(tmp[0] == '#' || !strlen(tmp))
			continue;

		vchatrc_parseargs(tmp);
	}

	fclose(vf);

	if(fx)
		printf("- done.\n");
	else
		printf("-done.\n");

}

void vchatrc_parseargs(char *arg)
{
	char *xargv[10];
	char **aptr;
	int xargc = 0;

#if HAVE_STRSEP
	for(aptr = xargv; (*aptr = (char *) strsep(&arg, " \t")) != NULL;)
	{
		if(**aptr != '\0')
		{
			++aptr;
			++xargc;
		}
	}
#else
	aptr = xargv;

	*aptr = (char *)strtok(arg, " \t");

	while(*aptr != NULL)
	{
		++aptr;
		++xargc;

		*aptr = (char *)strtok(NULL, " \t");
	}
#endif

	if(!xargc)
		return;

	if(!strcmp(xargv[0], "set"))
	{
		vchatrc_setvar(xargv[1], xargv[2]);
	}
	else if(!strcmp(xargv[0], "unset"))
	{
		vchatrc_unsetvar(xargv[1]);
	}
	else
	{
		printf("%sUnknown keyword '%s' - ignored.\n", fx ? "\n-" : "-", xargv[0]); 
	}
}

void vchatrc_setvar(char *s1, char *s2)
{
	struct hostent *ht;
	int mode;

	if(s1 == NULL)
	{
		printf("%smissing variable name - ignored.\n", fx ? "\n-" : "-");
		fx = 0;
		return;
	}

	if(!strcmp(s1, "nickname"))
	{
		if(s2 == NULL || !strlen(s2))
		{
			printf("%smissing argument for 'nickname' - ignored.\n", fx ? "\n-" : "-");
			fx = 0;
			return;
		}

		strncpy(vp.nick, s2, sizeof(vp.nick));
	}
	else if(!strcmp(s1, "vserver"))
	{
		if((ht = gethostbyname(s2)) == NULL)
		{
#if HAVE_HSTRERROR
			printf("%s%s: %s\n", fx ? "\n-" : "-", s2,
				hstrerror(h_errno));
#else
			printf("%s%s: hostname lookup failure\n", fx ? "\n-" : "-", s2);
#endif
			printf("-defaulting to vserver at %s [%s]\n", iptoname(vp.sv_ip), inet_ntoa(vp.sv_ip));
			fx = 0;
		}
		else
		{
			BCOPY(ht->h_addr, &vp.sv_ip, ht->h_length);
		}
	}
	else if(!strcmp(s1, "bs-mode"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'bs-mode' - defaulted.\n", fx ? "\n-" : "-");
			fx = 0;
			vp.bs_mode = 0;
		}
		else
		{
			if(!strcmp(s2, "move"))
			{
				vp.bs_mode = 0;
			}
			else if(!strcmp(s2, "scroll"))
			{
				vp.bs_mode = 1;
			}
			else
			{
				printf("%sInvalid value for option 'bs-mode' - possible values are 'move' and 'scroll'.\n", fx ? "\n-" : "-");
				printf("-Bs-mode defaulted.\n");

				fx = 0;
				vp.bs_mode = 0;
			}
		}
	}
	else if(!strcmp(s1, "scroll-mode"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'scroll-mode' - defaulted.\n", fx ? "\n-" : "-");
			fx = 0;
			vp.sc_mode = 0;
		}
		else
		{
			if(!strcmp(s2, "char"))
			{
				vp.sc_mode = 0;
			}
			else if(!strcmp(s2, "block"))
			{
				vp.sc_mode = 1;
			}
			else
			{
				printf("%sInvalid value for option 'scroll-mode' - possible values are 'char' and 'block'.\n", fx ? "\n-" : "-");
				printf("-Scroll-mode defaulted.\n");

				fx = 0;
				vp.sc_mode = 0;
			}
		}
	}
	else if(!strcmp(s1, "word-wrap"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'word-wrap' - defaulted.\n", fx ? "\n-" : "-");
			fx = 0;
			vp.word_wrap = 1;
		}
		else
		{
			if(!strcmp(s2, "on"))
			{
				vp.word_wrap = 1;
			}
			else if(!strcmp(s2, "off"))
			{
				vp.word_wrap = 0;
			}
			else
			{
				printf("%sInvalid value for option 'word-wrap' - possible values are 'on' and 'off'.\n", fx ? "\n-" : "-");
				printf("-Word-Wrap defaulted.\n");

				fx = 0;
				vp.word_wrap = 1;
			}
		}
	}
	else if(!strcmp(s1, "indent"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'indent' - defaulted.\n", fx ? "\n-" : "-");
			fx = 0;
			vp.indent = 1;
		}
		else
		{
			if(!strcmp(s2, "on"))
			{
				vp.indent = 1;
			}
			else if(!strcmp(s2, "off"))
			{
				vp.indent = 0;
			}
			else
			{
				printf("%sInvalid value for option 'indent' - possible values are 'on' and 'off'.\n", fx ? "\n-" : "-");
				printf("-Indent defaulted.\n");

				fx = 0;
				vp.indent = 1;
			}
		}
	}
	else if(!strcmp(s1, "hicolor"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'hicolor' - defaulted.\n", fx ? "\n-" : "-");

			fx = 0;
			vp.hicolor = 1;
		}
		else
		{
			if(!strcmp(s2, "on"))
			{
				vp.hicolor = 1;
			}
			else if(!strcmp(s2, "off"))
			{
				vp.hicolor = 0;
			}
			else
			{
				printf("%sInvalid value for option 'hicolor' - possible values are 'on' and 'off'.\n", fx ? "\n-" : "-");
				printf("-Hicolor defaulted.\n");

				fx = 0;
				vp.hicolor = 1;
			}
		}
	}
	else if(!strcmp(s1, "cursordelay"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'cursordelay' - defaulted.\n", fx ? "\n-" : "-");

			fx = 0;
			vp.cursordelay = 3;
		}
		else
		{
			if((mode = atoi(s2)) < 0x1 || mode > 0xff)
			{
				printf("%sValue for option 'cursordelay' out of range - Valid values range from 1 to 255.\n", fx ? "\n-" : "-");
				printf("-Cursordelay defaulted.\n");

				fx = 0;
				vp.cursordelay = 3;
			}
			else
			{
				vp.cursordelay = mode;
			}
		}
	}
	else if(!strcmp(s1, "videosnake"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'videosnake' - defaulted.\n", fx ? "\n-" : "-");

			fx = 0;
			vp.videosnake = 0;
		}
		else
		{

			if((mode = atoi(s2)) < 0x1 || mode > 0xff)
			{
				printf("%sValue for option 'videosnake' out of range - Valid values range from 1 to 255.\n", fx ? "\n-" : "-");
				printf("-Videosnake defaulted.\n");

				fx = 0;
				vp.videosnake = 0;
			}
			else
			{
				vp.videosnake = mode;
			}
		}
	}
	else if(!strcmp(s1, "history"))
	{
		if(s2 == NULL)
		{
			printf("%sMissing argument for option 'history' - defaulted.\n", fx ? "\n-" : "-");

			fx = 0;
			vp.history_lines = HISTORY_LINES;
		}
		else
		{

			if((mode = atoi(s2)) < 0 || mode > 250)
			{
				printf("%sValue for option 'history' out of range - Valid values range from 0 to 250.\n", fx ? "\n-" : "-");
				printf("-History defaulted.\n");

				fx = 0;
				vp.history_lines = HISTORY_LINES;
			}
			else
			{
				vp.history_lines = mode;
			}
		}
	}
	else if(!strcmp(s1, "matchbeep"))
	{
		if(s2 == NULL)
		{
			vp.matchbeep = 1;
		}
		else
		{
			if(!strcmp(s2, "never") || !strcmp(s2, "off"))
			{
				vp.matchbeep = 0;
			}
			else if(!strcmp(s2, "ambigious") || !strcmp(s2, "on"))
			{
				vp.matchbeep = 1;
			}
			else
			{
				printf("Invalid argument for option 'matchbeep'\n");
				printf("-Matchbeep defaulted.\n");
				vp.matchbeep = 1;
			}
		}
	}
	else
	{
		printf("%sUnknown variable name '%s' - ignored.\n", fx ? "\n-" : "-", s1);
		fx = 0;
	}
}

void vchatrc_unsetvar(char *s1)
{
	if(s1 == NULL)
	{
		printf("%sMissing variable name - ignored.\n", fx ? "\n-" : "-");
		fx = 0;
		return;
	}

	if(!strcmp(s1, "nickname"))
	{
		strcpy(vp.nick, vp.user);
	}
	else if(!strcmp(s1, "vserver"))
	{
		/* Ignored for now */
	}
	else if(!strcmp(s1, "bs-mode"))
	{
		vp.bs_mode = 0;
	}
	else if(!strcmp(s1, "scroll-mode"))
	{
		vp.sc_mode = 0;
	}
	else if(!strcmp(s1, "word-wrap"))
	{
		vp.word_wrap = 0;
	}
	else if(!strcmp(s1, "indent"))
	{
		vp.indent = 0;
	}
	else if(!strcmp(s1, "hicolor"))
	{
		vp.hicolor = 0;
	}
	else if(!strcmp(s1, "cursordelay"))
	{
		vp.cursordelay = 3;
	}
	else if(!strcmp(s1, "videosnake"))
	{
		vp.videosnake = 0;
	}
	else if(!strcmp(s1, "history"))
	{
		vp.history_lines = 0;
	}
	else if(!strcmp(s1, "matchbeep"))
	{
		
		vp.matchbeep = 0;
	}
	else
	{
		printf("%sUnknown variable name '%s' - ignored.\n", fx ? "\n-" : "-", s1);
		fx = 0;
	}
}
