/******************************************************************************
 * This file is part of a software distribution, which is furnished under the *
 * terms of a license.  Use of this software  by any means is subject to this *
 * license  and  signifies  the  acceptance of  the  licensing  terms  stated *
 * therein. Please see  the file LICENSE in the  top-level directory  of this *
 * software  distribution  for detailed copyright  disclaimers  and licensing *
 * terms.                                                                     *
 ******************************************************************************
 * Copryight (c) by Andreas S. Wetzel - All rights reserved.                  *
 ******************************************************************************/

/* $Id: vd_telsubr.c,v 1.2 2001/03/19 14:54:06 mickey Exp $ */

#include <vchat.h>
#include <if_telnet.h>
#include <proto_vchatd.h>

#include <netdb.h>
#include <ctype.h>
#include <sys/socket.h>
#include <arpa/inet.h>

/*** Globals ***/

extern	char hostname[];
extern	u_char subopt_buf[];

extern	u_short tel_option[];
extern	u_short	do_dont_response[];
extern	u_short	will_wont_response[];

extern	char *prog_name;
extern	char *bannerfile;

extern	u_char *sub_bufptr;
extern	u_char *sub_endptr;

extern	struct in_addr our_ipaddr;
extern	struct sockaddr_in tel_sock_in;

extern	VP vp;
extern	VD vd;

/*** Code ***/

int init_telnet_conn(void)
{
	int			tel_sock_in_len	= 0;

	struct	sockaddr_in	local_addr;
	int			addrsize	= sizeof(struct sockaddr_in);
	int			on		= 1;

	/*
	 * Get local address for the connected peer
	 */

	if(GETSOCKNAME(0, &local_addr, &addrsize) == -1)
	{
		log(VLOG_ERR, "unable to get local address for connected peer (%s)",
			strerror(errno));
		return(-1);
	}

	vp.my_ip = local_addr.sin_addr;

	/* Initialize socket options and */
	/* data structures.              */

	BZERO((char *)&tel_sock_in, sizeof(tel_sock_in));

	tel_sock_in_len = sizeof(tel_sock_in);

	if(setsockopt(0, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on)))
	{
		perror(prog_name);
		return(-1);
	}

	/* Now get the name of the connected */
	/* peer we are currently working on  */
	/* and find out who connected us.    */

	if(GETPEERNAME(0, &tel_sock_in, &tel_sock_in_len) < 0)
	{
		perror(prog_name);
		return(-1);
	}

	vd.cl_ip = tel_sock_in.sin_addr;

	log(VLOG_INFO, "Client => %s [%s].", inet_ntoa(vd.cl_ip), iptoname(vd.cl_ip));

	return(0);
}

int query_nickname(void)
{
	u_char count = 0;

	/*
	 * Get nickname for this VChat session
	 */

	strcpy(vp.home, "\0");

	while(++count <= 5)
	{
		printf("Enter your nickname: ");

		if(get_user_input((char *)&vp.nick, VPNICKSIZE, 1) == -1)
			return(-1);

		if(strlen(vp.nick))
			break;
		else
			printf("\r\n");
	}

	if(count > 5)
	{
		printf("\r\n\r\nToo many tries, giving up...\r\n\r\n");
		return(-1);
	}
	else
	{
		printf("\r\n");
	}

	return(0);
}

int get_user_input(char *buf, size_t size, char doecho)
{
	static	char	lterm	= 0;
	int		cnt	= 0;
	char		c;

	for(;;)
	{
		/*
		 * Read character from input stream
		 */

		switch(read(0, &c, 1))
		{
			case  1:	break;
			case  0:
			case -1:	printf("\r\n");
					log(VLOG_ERR, "TELNET connection closed by remote host");
					return(-1);
			default:	printf("\r\n");
					log(VLOG_ERR, "OOPS! read() returned invalid value - exiting.");
					return(-1);
		}

		/*
		 * Process character
		 */

		if(c == EOT)
		{
			printf("\r\n");
			log(VLOG_ERR, "Connection closed by remote host");
			return(-1);
		}
		else if((c == '\n' && lterm != '\r')
		|| (c == '\r' && lterm != '\n'))
		{
			lterm = c;
			break;
		}
		else if(isprint((int)c) && !isspace((int)c) && cnt < size)
		{
			*buf++ = c;
			++cnt;

			if(doecho)
				putchar(c);
		}
		else if(iscntrl((int)c))
		{
			switch(c)
			{
				case DEL:
				case BS:	if(cnt)
						{
							*(--buf) = '\0';
							--cnt;

							if(doecho)
								printf("%c%c%c", BS, ' ', BS);
						}
						break;

				case NAK:	while(cnt)
						{
							*(--buf) = '\0';
							--cnt;

							if(doecho)
								printf("%c%c%c", BS, ' ', BS);
						}
						break;
			}
		}
	}

	*buf++ = '\0';

	return(cnt);
}

int negotiate_telopts(void)
{
	send_will(TELOPT_SGA, 0);
	send_will(TELOPT_ECHO, 0);

	while(his_do_dont_differs(TELOPT_SGA)
	|| his_do_dont_differs(TELOPT_ECHO))
	{
		if(cmd_loop() == -1)
			return(-1);
	}

	return(0);
}

void show_banner(void)
{
	char buf[256];
	char *x;
	FILE *fd;

	if(bannerfile)
	{
		if((fd = fopen(bannerfile, "r")) == NULL)
		{
			fprintf(stderr, "%s: failed to open bannerfile '%s' (%s)\r\n",
				prog_name, bannerfile, strerror(errno));
		}
		else
		{
			while(fgets(buf, 256, fd) != NULL)
			{
				while((x = (char *)strrchr(buf, '\n')) != NULL)
					*x = '\0';

				while((x = (char *)strrchr(buf, '\r')) != NULL)
					*x = '\0';

				printf("%s\r\n", buf);
			}

			fclose(fd);
		}
	}
	else
	{
		printf("\r\n**** Welcome to VChat conference gateway at %s ****\r\n",
			vd.my_host);
	}

	printf("\r\nvchat conference TCP gateway (vchatd %s - %s) ready.\r\n", VERSION, REL_DATE);
}

void init_tel_opts(void)
{
	int i;

	for(i = 0; i < 256; i++)
		tel_option[i] = do_dont_response[i] = will_wont_response[i] = 0;

	vd.resize_ok = 0x00;
}

u_char tel_filter(u_char c)
{
	static int mode = MODE_DATA;

	switch(mode)
	{
		case MODE_DATA:	if(c == IAC)
				{
					mode = MODE_IAC;
				}
				else
				{
					return(c);
				}	
				break;
		case MODE_IAC:	switch(c)
				{
					case IAC:	mode = MODE_DATA;
							return(IAC);
							break;
					case DONT:	mode = MODE_DONT;
							break;
					case DO:	mode = MODE_DO;
							break;
					case WONT:	mode = MODE_WONT;
							break;
					case WILL:	mode = MODE_WILL;
							break;
					case SB:	SUB_CLEAR();
							mode = MODE_SB;
							break;
					case GA:	mode = MODE_DATA;
							break;
					case EL:	mode = MODE_DATA;
							return(NAK);
							break;
					case EC:	mode = MODE_DATA;
							return(BS);
							break;
					case AYT:	mode = MODE_DATA;
							printf("\r\nI am still alive!\r\n");
							break;
					case AO:	mode = MODE_DATA;
#if HAVE_FPURGE
							fpurge(stdout);
#endif
							break;
					case IP:	log(VLOG_INFO, "Received IP signal");
							vquit("Received IP signal.");
							break;
					case BREAK:	mode = MODE_DATA;
							printf("\r\n*** BREAK ***\r\n");
							break;

					case DM:	mode = MODE_DATA;
							/*
							 * Not handled yet
							 */
							break;
					case NOP:	mode = MODE_DATA;
							/* No Operation */
							break;
					case SE:	mode = MODE_SE;
							break;
					case EOR:
					case ABORT:
					case SUSP:	/*** NOT HANDLED ***/
					case xEOF:	mode = MODE_DATA;
							break;
					default:	mode = MODE_DATA;
							break;
				}
				break;
		case MODE_SB:	if(c == IAC)
					mode = MODE_SE;
				else
					SUB_STORE(c);
				break;
		case MODE_SE:	if(c == SE)
				{
					/* Everything is fine, and our */
					/* suboption ends here.        */

					SUB_FLUSH();
					handle_suboption();
					mode = MODE_DATA;
				}
				else if(c == IAC)
				{
					SUB_STORE(c);
					mode = MODE_SB;
				}
				else
				{
					/* Something got wrong?! */
					/* But try to handle it  */

					SUB_FLUSH();
					handle_suboption();
					mode = MODE_IAC;
					return(c);
				}
				break;
		case MODE_WILL:	willoption(c);
				mode = MODE_DATA;
				break;
		case MODE_WONT:	wontoption(c);
				mode = MODE_DATA;
				break;
		case MODE_DO:	dooption(c);
				mode = MODE_DATA;
				break;
		case MODE_DONT:	dontoption(c);
				mode = MODE_DATA;
				break;
		default:	log(VLOG_ERR, "unknown mode in tel_filter();");
				break;
	}
	return(0);
}

int check_termtype(char *s)
{
	char buf[1024];

	return(tgetent((char *)&buf, s));
}
